<?php

namespace Drupal\google_cloud_logging;

use DateTimeImmutable as DateTime;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Queue\QueueFactory;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Config\ImmutableConfig;

/**
 * A PSR-3 logger that logs Drupal log entries to Google Cloud Logging.
 */
class Logger implements LoggerInterface {

  use RfcLoggerTrait;

  /**
   * Work queue.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Module configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs a new object.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queueFactory
   *   Queue factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request stack.
   */
  public function __construct(QueueFactory $queueFactory, RequestStack $requestStack, ConfigFactoryInterface $configFactory) {
    $this->queue = $queueFactory->get('google_cloud_logging');
    $this->requestStack = $requestStack;
    $this->config = $configFactory->get('google_cloud_logging.settings');
  }

  /**
   * Write a log entry to the background queue.
   *
   * @param string|int $level Severity of the log entry.
   * @param string $message Message to log.
   * @param array $context Context data.
   */
  public function log($level, $message, array $context = []) {
    if(!isset($this->config->get('log_levels')[$level]) || $this->config->get('log_levels')[$level] != $level) {
      return;
    }
    if ($context['channel'] === 'google_cloud_logging' || in_array($context['channel'], $this->config->get('ignored_channels') ?: [])) {
      return;
    }

    if ($request = ($this->requestStack->getCurrentRequest() ?? NULL)) {
      $request = [
        'method' => $request->getMethod(),
        'url' => $request->getUri(),
        'userAgent' => $request->headers->get('user-agent'),
        'remoteIp' => $request->getClientIP(),
        'protocol' => $request->getProtocolVersion(),
        'referer' => $request->headers->get('Referer', '') ?? NULL,
      ];
    }

    foreach (debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS) as $frame) {
      if ((empty($frame['class']) || $frame['class'] !== get_class($this) && strpos($frame['class'], 'Drupal\\Core\\Logger\\') !== 0) && !empty($frame['file'])) {
        $caller = [
          'file' => $frame['file'],
          'line' => intval($frame['line'] ?? 0),
          'function' => (isset($frame['class']) ? $frame['class'] . '->' : '') . $frame['function'] . '()',
        ];
        break;
      }
    }

    $this->queue->createItem([
      'level' => $level,
      'message' => (string)$message,
      'context' => (new NormalizerFormatter())->format($context),
      'request' => $request,
      'caller' => $caller ?? NULL,
      'time' => new DateTime(),
    ]);
  }

}
